@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2021, 2022, 2023 Omar Polo <op@omarpolo.com>
+ * Copyright (c) 2020-2024 Omar Polo <op@omarpolo.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -86,6 +86,15 @@
 
 #define TLS_CERT_HASH_SIZE	128
 
+/*
+ * glibc is violating POSIX by defining HOST_NAME_MAX to a ridicully
+ * small value, so we can't use it.  Luckily, we don't have to do DNS
+ * so we don't risk to pass buffers too big to functions that might
+ * not expect them, we just need a fixed size buffer to catch possible
+ * misconfigurations.
+ */
+#define GMID_HOST_NAME_MAX	255	/* without NUL */
+
 /* forward declaration */
 struct privsep;
 struct privsep_proc;
@@ -113,6 +122,7 @@ struct address {
 	struct sockaddr_storage	 ss;
 	socklen_t		 slen;
 	int16_t			 port;
+	int			 proxy;
 
 	/* pretty-printed version of `ss' */
 	char			 pp[NI_MAXHOST];
@@ -143,19 +153,19 @@ struct envlist {
 
 TAILQ_HEAD(aliashead, alist);
 struct alist {
-	char		alias[HOST_NAME_MAX + 1];
+	char		alias[GMID_HOST_NAME_MAX + 1];
 	TAILQ_ENTRY(alist) aliases;
 };
 
 TAILQ_HEAD(proxyhead, proxy);
 struct proxy {
 	char		 match_proto[32];
-	char		 match_host[HOST_NAME_MAX + 1];
+	char		 match_host[GMID_HOST_NAME_MAX + 1];
 	char		 match_port[32];
 
-	char		 host[HOST_NAME_MAX + 1];
+	char		 host[GMID_HOST_NAME_MAX + 1];
 	char		 port[32];
-	char		 sni[HOST_NAME_MAX];
+	char		 sni[GMID_HOST_NAME_MAX];
 	int		 notls;
 	uint32_t	 protocols;
 	int		 noverifyname;
@@ -167,6 +177,7 @@ struct proxy {
 	size_t		 keylen;
 	char		*reqca_path;
 	X509_STORE	*reqca;
+	int		 proxy;	/* use the proxy protocol */
 
 	TAILQ_ENTRY(proxy) proxies;
 };
@@ -197,7 +208,7 @@ struct location {
 
 TAILQ_HEAD(vhosthead, vhost);
 struct vhost {
-	char		 domain[HOST_NAME_MAX + 1];
+	char		 domain[GMID_HOST_NAME_MAX + 1];
 	char		*cert_path;
 	char		*key_path;
 	char		*ocsp_path;
@@ -281,17 +292,44 @@ enum {
 	REQUEST_DONE,
 };
 
+enum proto {
+	PROTO_V4,
+	PROTO_V6,
+	PROTO_UNKNOWN,
+};
+
+struct proxy_protocol_v1 {
+	enum proto		 proto;
+	struct sockaddr_storage	 srcaddr;
+	socklen_t		 srclen;
+	struct sockaddr_storage	 dstaddr;
+	socklen_t		 dstlen;
+	uint16_t		 srcport;
+	uint16_t		 dstport;
+};
+
+#define BUFLAYER_MAX 108
+
+struct buflayer {
+	char data[BUFLAYER_MAX];
+	size_t len;
+	size_t read_pos;
+	int has_tail;
+};
+
 struct client {
-	struct conf	*conf;
-	struct address	*addr;
-	uint32_t	 id;
-	struct tls	*ctx;
-	char		*req;
-	size_t		 reqlen;
-	struct iri	 iri;
-	char		 domain[DOMAIN_NAME_LEN];
-	char		 rhost[NI_MAXHOST];
-	char		 rserv[NI_MAXSERV];
+	struct conf		*conf;
+	struct address		*addr;
+	int 			 proxy_proto;
+	struct buflayer		 buf;
+	uint32_t		 id;
+	struct tls		*ctx;
+	char			*req;
+	size_t			 reqlen;
+	struct iri		 iri;
+	char			 domain[DOMAIN_NAME_LEN];
+	char			 rhost[NI_MAXHOST];
+	char			 rserv[NI_MAXSERV];
 
 	struct bufferevent *bev;
 
@@ -464,4 +502,8 @@ struct vhost	*new_vhost(void);
 struct location	*new_location(void);
 struct proxy	*new_proxy(void);
 
+/* proxy-proto.c */
+int proxy_proto_v1_parse(struct proxy_protocol_v1 *, char *, size_t, size_t *);
+int proxy_proto_v1_string(const struct proxy_protocol_v1 *, char*, size_t);
+
 #endif
