@@ -90,6 +90,35 @@ test_static_files() {
 	check_reply "20 text/gemini" "# hello world" || return 1
 }
 
+test_alias() {
+	alias="foo.example"
+	setup_simple_test '' "alias '$alias'"
+	gghost="$alias"
+	ggflags="-P localhost:$port -H $alias"
+
+	fetch /
+	check_reply "20 text/gemini" "# hello world" || return 1
+}
+
+test_alias_long_hostname() {
+	# longer than 64 chars to detect issues with glibc
+	# posix-violating tiny HOST_NAME_MAX.
+	alias=foo.jjijhrngx5ZxWY0p3o9jag3fOEuZQ64Iuler243oAkkCCqq8pFufw43DBLgVX.test
+
+	setup_simple_test '' "alias '$alias'"
+
+	fetch /
+	check_reply "20 text/gemini" "# hello world" || return 1
+
+	# now the same but against the alias
+
+	gghost=$alias
+	ggflags="-P localhost:$port -H $alias"
+
+	fetch /
+	check_reply "20 text/gemini" "# hello world" || return 1
+}
+
 test_directory_redirect() {
 	setup_simple_test
 
@@ -277,12 +306,10 @@ test_fastcgi() {
 
 	setup_simple_test 'prefork 1' 'fastcgi socket "'$PWD'/fcgi.sock"'
 
-	msg=$(printf "# hello from fastcgi!\nsome more content in the page...")
-
 	i=0
 	while [ $i -lt 10 ]; do
 		fetch /
-		check_reply "20 text/gemini" "$msg"
+		check_reply "20 text/gemini" "$(fcgi_content)"
 		if [ $? -ne 0 ]; then
 			kill $fcgi_pid
 			return 1
@@ -298,15 +325,16 @@ test_fastcgi() {
 test_fastcgi_inside_location() {
 	./fcgi-test fcgi.sock &
 	fcgi_pid=$!
+	fcgi_path_info="/foo"
+	fcgi_path="$PWD/testdata/foo"
 
 	setup_simple_test 'prefork 1' 'fastcgi socket "'$PWD'/fcgi.sock"
 	location "/dir/*" {
 		fastcgi off
 	}'
 
-	msg=$(printf "# hello from fastcgi!\nsome more content in the page...")
 	fetch /foo
-	if ! check_reply "20 text/gemini" "$msg"; then
+	if ! check_reply "20 text/gemini" "$(fcgi_content)"; then
 		kill $fcgi_pid
 		return 1
 	fi
@@ -324,6 +352,8 @@ test_fastcgi_inside_location() {
 test_fastcgi_location_match() {
 	./fcgi-test fcgi.sock &
 	fcgi_pid=$!
+	fcgi_path_info="/foo"
+	fcgi_path="$PWD/testdata/foo"
 
 	setup_simple_test 'prefork 1' '
 	location "/dir/*" {
@@ -333,9 +363,8 @@ test_fastcgi_location_match() {
 		fastcgi socket "'$PWD'/fcgi.sock"
 	}'
 
-	msg=$(printf "# hello from fastcgi!\nsome more content in the page...")
 	fetch /foo
-	if ! check_reply "20 text/gemini" "$msg"; then
+	if ! check_reply "20 text/gemini" "$(fcgi_content)"; then
 		kill $fcgi_pid
 		return 1
 	fi
@@ -358,9 +387,8 @@ test_fastcgi_deprecated_syntax() {
 	# backward compatibility works.
 	setup_simple_test 'prefork 1' 'fastcgi "'$PWD'/fcgi.sock"'
 
-	msg=$(printf "# hello from fastcgi!\nsome more content in the page...")
 	fetch /
-	check_reply "20 text/gemini" "$msg"
+	check_reply "20 text/gemini" "$(fcgi_content)"
 	if [ $? -ne 0 ]; then
 		kill $fcgi_pid
 		return 1
@@ -402,14 +430,14 @@ test_proxy_relay_to() {
 
 	run
 
-	ggflags="-P localhost:$port -H localhost.local"
+	ggflags="-P localhost:$proxy_port -H localhost.local"
 
 	fetch /
 	check_reply "20 text/gemini" "# hello world"
 }
 
 test_proxy_with_certs() {
-	ggflags="-P localhost:$port -H localhost.local"
+	ggflags="-P localhost:$proxy_port -H localhost.local"
 
 	# first test using the valid keys
 
@@ -521,3 +549,44 @@ test_ipv6_server() {
 	fetch /
 	check_reply "20 text/gemini" "# hello world" || return 1
 }
+
+test_high_prefork() {
+	quit
+
+	setup_simple_test 'prefork 12'
+
+	fetch /
+	check_reply "20 text/gemini" "# hello world" || return 1
+
+	dont_check_server_alive=yes
+	quit
+}
+
+test_proxy_protocol_v1() {
+	rm -f log
+	proxy=proxy-v1
+	gen_config '
+log access "'$PWD'/log"
+log style legacy' ''
+	set_proxy 'proxy-v1'
+	run
+
+	ggflags="-P localhost:$proxy_port -H localhost.local"
+
+	# This will generate two log entry: one for the "frontend"
+	# and one for the proxied request.  Both should have exactly
+	# the same IP and port.
+	fetch_hdr /
+	check_reply '20 text/gemini'
+
+	n=$(awk '{print $1}' log | uniq | wc -l)
+	if [ "$n" -ne 1 ]; then
+		# keep the log for post-mortem analysis
+		echo
+		echo "n is $n"
+		return 1
+	fi
+
+	rm -f log
+	return 0
+}
