@@ -19,7 +19,6 @@
 #include <sys/stat.h>
 #include <sys/un.h>
 
-#include <assert.h>
 #include <ctype.h>
 #include <errno.h>
 #include <event.h>
@@ -78,6 +77,9 @@ static int	 server_dispatch_parent(int, struct privsep_proc *, struct imsg *);
 static int	 server_dispatch_crypto(int, struct privsep_proc *, struct imsg *);
 static int	 server_dispatch_logger(int, struct privsep_proc *, struct imsg *);
 
+static ssize_t read_cb(struct tls *, void *, size_t, void *);
+static ssize_t write_cb(struct tls *, const void *, size_t, void *);
+
 static struct privsep_proc procs[] = {
 	{ "parent",	PROC_PARENT,	server_dispatch_parent },
 	{ "crypto",	PROC_CRYPTO,	server_dispatch_crypto },
@@ -111,22 +113,6 @@ static inline int
 match_host(struct vhost *v, struct client *c)
 {
 	struct alist *a;
-	struct address *addr;
-
-	TAILQ_FOREACH(addr, &v->addrs, addrs)
-		if (match_addr(addr, c->addr))
-			break;
-	if (addr == NULL)
-		return 0;
-
-	if (*c->domain == '\0') {
-		if (strlcpy(c->domain, addr->pp, sizeof(c->domain))
-		    >= sizeof(c->domain)) {
-			log_warnx("%s: domain too long: %s", __func__,
-			    addr->pp);
-			*c->domain = '\0';
-		}
-	}
 
 	if (matches(v->domain, c->domain))
 		return 1;
@@ -416,11 +402,15 @@ handle_handshake(int fd, short ev, void *d)
 		return;
 	}
 
-	/*
-	 * match_addr will serialize the (matching) address if c->domain
-	 * is empty, so that we can support requests for raw IPv6 address
-	 * that can't have a SNI.
-	 */
+	if (*c->domain == '\0') {
+		if (strlcpy(c->domain, c->addr->pp, sizeof(c->domain))
+		    >= sizeof(c->domain)) {
+			log_warnx("%s: domain too long: %s", __func__,
+			    c->addr->pp);
+			*c->domain = '\0';
+		}
+	}
+
 	TAILQ_FOREACH(h, &conf->hosts, vhosts)
 		if (match_host(h, c))
 			break;
@@ -554,21 +544,23 @@ check_matching_certificate(X509_STORE *store, struct client *c)
 }
 
 static int
-proxy_socket(struct client *c, const char *host, const char *port)
+proxy_socket(struct client *c, struct proxy *p)
 {
 	struct addrinfo hints, *res, *res0;
 	int r, sock, save_errno;
 	const char *cause = NULL;
+	char to[NI_MAXHOST], to_port[NI_MAXSERV];
+	int err;
 
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 
 	/* XXX: asr_run? :> */
-	r = getaddrinfo(host, port, &hints, &res0);
+	r = getaddrinfo(p->host, p->port, &hints, &res0);
 	if (r != 0) {
 		log_warnx("getaddrinfo(\"%s\", \"%s\"): %s",
-		    host, port, gai_strerror(r));
+		    p->host, p->port, gai_strerror(r));
 		return -1;
 	}
 
@@ -593,10 +585,31 @@ proxy_socket(struct client *c, const char *host, const char *port)
 	}
 
 	if (sock == -1)
-		log_warn("can't connect to %s:%s: %s", host, port, cause);
+		log_warn("can't connect to %s:%s: %s", p->host, p->port, cause);
+
+	if (res && sock != -1 && p->proxy) {
+		err = getnameinfo(res->ai_addr, res->ai_addrlen,
+		    to, sizeof(to), to_port, sizeof(to_port),
+		    NI_NUMERICHOST|NI_NUMERICSERV);
+		if (err != 0) {
+			log_warnx("%s: getnameinfo failed: %s", __func__,
+			    gai_strerror(err));
+			strlcpy(to, c->rhost, sizeof(to));
+			strlcpy(to_port, c->rserv, sizeof(to_port));
+		}
+
+		r = snprintf(c->buf.data, sizeof(c->buf.data),
+		    "PROXY TCP%c %s %s %s %s\r\n",
+		    c->addr->ai_family == AF_INET ? '4' : '6',
+		    c->rhost, to, c->rserv, to_port);
+		if (r < 0 || (size_t)r >= sizeof(c->buf.data)) {
+			log_warnx("failed serialize info for the proxy protocol");
+			c->buf.data[0] = '\0';
+		} else
+			c->buf.len = r;
+	}
 
 	freeaddrinfo(res0);
-
 	return sock;
 }
 
@@ -616,7 +629,7 @@ apply_reverse_proxy(struct client *c)
 
 	log_debug("opening proxy connection for %s:%s", p->host, p->port);
 
-	if ((c->pfd = proxy_socket(c, p->host, p->port)) == -1) {
+	if ((c->pfd = proxy_socket(c, p)) == -1) {
 		start_reply(c, PROXY_ERROR, "proxy error");
 		return 1;
 	}
@@ -1007,7 +1020,7 @@ client_read(struct bufferevent *bev, void *d)
 
 	/* ignore the port number */
 	if (strcmp(c->iri.schema, "gemini") ||
-	    strcmp(decoded, c->domain)) {
+	    strcasecmp(c->domain, decoded)) {
 		start_reply(c, PROXY_REFUSED, "won't proxy request");
 		return;
 	}
@@ -1292,6 +1305,102 @@ client_close(struct client *c)
 	client_close_ev(c->fd, 0, c);
 }
 
+static ssize_t
+read_cb(struct tls *ctx, void *buf, size_t buflen, void *cb_arg)
+{
+	struct client		*c = cb_arg;
+	struct proxy_protocol_v1 pp1 = {0};
+	char			 protostr[1024];
+	ssize_t			 ret;
+	size_t			 left, avail, copy, consumed;
+	int			 err, status;
+
+	if (!c->proxy_proto) {
+		/* no buffer to cache into, read into libtls buffer */
+		ret = read(c->fd, buf, buflen);
+		if (ret == -1 && errno == EWOULDBLOCK)
+			ret = TLS_WANT_POLLIN;
+
+		return ret;
+	}
+
+	if (c->buf.has_tail) {
+		/* we have leftover data from a previous call to read_cb */
+		left = c->buf.len - c->buf.read_pos;
+		copy = MINIMUM(buflen, left);
+		memcpy(buf, c->buf.data + c->buf.read_pos, copy);
+		c->buf.read_pos += copy;
+		if (left == copy) {
+			c->proxy_proto = 0;
+			c->buf.has_tail = 0;
+		}
+		return copy;
+	}
+
+	avail = sizeof(c->buf.data) - c->buf.len - 1; /* for a NUL */
+	if (avail == 0) {
+		log_warnx("read_cb: overlong proxy protocol v1 header");
+		return -1;
+	}
+
+	ret = read(c->fd, c->buf.data + c->buf.len, avail);
+	if (ret == -1 && errno == EWOULDBLOCK)
+		return TLS_WANT_POLLIN;
+	if (ret <= 0)
+		return ret;
+	c->buf.len += ret;
+
+	if (memmem(c->buf.data, c->buf.len, "\r\n", 2) == NULL)
+		return TLS_WANT_POLLIN;
+
+	status = proxy_proto_v1_parse(&pp1, c->buf.data, c->buf.len, &consumed);
+	if (status == -1) {
+		log_warnx("read_cb: received invalid proxy protocol header");
+		return -1;
+	}
+
+	if (pp1.proto == PROTO_UNKNOWN)
+		strlcpy(c->rhost, "UNKNOWN", sizeof(c->rhost));
+	else {
+		err = getnameinfo((struct sockaddr *)&pp1.srcaddr, pp1.srclen,
+		    c->rhost, sizeof(c->rhost), NULL, 0,
+		    NI_NUMERICHOST);
+		if (err) {
+			log_warn("%s: getnameinfo failed: %s", __func__,
+			    gai_strerror(err));
+			return -1;
+		}
+		snprintf(c->rserv, sizeof(c->rserv), "%u", pp1.srcport);
+	}
+
+	proxy_proto_v1_string(&pp1, protostr, sizeof(protostr));
+	log_debug("proxy-protocol v1: %s", protostr);
+
+	if (consumed < c->buf.len) {
+		/* we have some leftover */
+		c->buf.read_pos = consumed;
+		c->buf.has_tail = 1;
+	} else {
+		/* we consumed the whole buffer */
+		c->proxy_proto = c->buf.read_pos = 0;
+		c->buf.has_tail = 0;
+	}
+
+	return TLS_WANT_POLLIN;
+}
+
+static ssize_t
+write_cb(struct tls *ctx, const void *buf, size_t buflen, void *cb_arg)
+{
+	struct client *c = cb_arg;
+
+	ssize_t ret = write(c->fd, buf, buflen);
+	if (ret == -1 && errno == EAGAIN)
+		return TLS_WANT_POLLOUT;
+
+	return ret;
+}
+
 void
 server_accept(int sock, short et, void *d)
 {
@@ -1322,17 +1431,19 @@ server_accept(int sock, short et, void *d)
 	c->pfd = -1;
 	memcpy(&c->raddr, &raddr, sizeof(raddr));
 	c->raddrlen = len;
+	c->proxy_proto = addr->proxy;
 
 	e = getnameinfo(sraddr, len, c->rhost, sizeof(c->rhost),
 	    c->rserv, sizeof(c->rserv), NI_NUMERICHOST | NI_NUMERICSERV);
 	if (e != 0) {
-		log_warnx("getnameinfo failed: %s", gai_strerror(e));
+		log_warnx("%s: getnameinfo failed: %s", __func__,
+		    gai_strerror(e));
 		close(c->fd);
 		free(c);
 		return;
 	}
 
-	if (tls_accept_socket(addr->ctx, &c->ctx, fd) == -1) {
+	if (tls_accept_cbs(addr->ctx, &c->ctx, read_cb, write_cb, c) == -1) {
 		log_warnx("failed to accept socket: %s", tls_error(c->ctx));
 		close(c->fd);
 		free(c);
